@@ -105,9 +105,14 @@ def run(image=None):
     warnings.filterwarnings("ignore")
     app = QtGui.QApplication(sys.argv)
     icon_path = pathlib.Path.home().joinpath('.cellpose', 'logo.png')
+    guip_path = pathlib.Path.home().joinpath('.cellpose', 'cellpose_gui.png')
     if not icon_path.is_file():
+        cp_dir = pathlib.Path.home().joinpath('.cellpose')
+        cp_dir.mkdir(exist_ok=True)
         print('downloading logo')
         utils.download_url_to_file('http://www.cellpose.org/static/images/cellpose_transparent.png', icon_path, progress=True)
+    if not guip_path.is_file():
+        utils.download_url_to_file('https://github.com/MouseLand/cellpose/raw/master/docs/_static/cellpose_gui.png', guip_path, progress=True)
     icon_path = str(icon_path.resolve())
     app_icon = QtGui.QIcon()
     app_icon.addFile(icon_path, QtCore.QSize(16, 16))
@@ -212,6 +217,10 @@ class MainW(QtGui.QMainWindow):
         HW = guiparts.HelpWindow(self)
         HW.show()
 
+    def gui_window(self):
+        EG = guiparts.ExampleGUI(self)
+        EG.show()
+
     def make_buttons(self):
         label_style = """QLabel{
                             color: white
@@ -352,9 +361,10 @@ class MainW(QtGui.QMainWindow):
         label = QtGui.QLabel('cell diameter (pixels) (click ENTER):')
         label.setStyleSheet(label_style)
         label.setFont(self.medfont)
-        label.setToolTip('you can manually enter the approximate diameter for your cells, or press “calibrate” to let the model estimate it. The size is represented by a disk at the bottom of the view window (can turn this disk of by unchecking “scale disk on”)')
+        label.setToolTip('you can manually enter the approximate diameter for your cells, \nor press “calibrate” to let the model estimate it. \nThe size is represented by a disk at the bottom of the view window \n(can turn this disk off by unchecking “scale disk on”)')
         self.l0.addWidget(label, b, 0,1,2)
         self.Diameter = QtGui.QLineEdit()
+        self.Diameter.setToolTip('you can manually enter the approximate diameter for your cells, \nor press “calibrate” to let the model estimate it. \nThe size is represented by a disk at the bottom of the view window \n(can turn this disk off by unchecking “scale disk on”)')
         self.Diameter.setText(str(self.diameter))
         self.Diameter.setFont(self.medfont)
         self.Diameter.returnPressed.connect(self.compute_scale)
@@ -391,12 +401,11 @@ class MainW(QtGui.QMainWindow):
         self.l0.addWidget(self.useGPU, b,0,1,1)
 
         # fast mode
-        self.FastMode = QtGui.QCheckBox('fast mode')
-        self.FastMode.setStyleSheet(self.checkstyle)
-        self.FastMode.setFont(self.medfont)
-        self.FastMode.setChecked(False)
-        self.FastMode.setToolTip('turn off augmentations + averaging 4 different fit networks to <i>increase</i> run speed')
-        self.l0.addWidget(self.FastMode, b,1,1,1)
+        self.NetAvg = QtGui.QComboBox()
+        self.NetAvg.addItems(['average 4 nets', 'run 1 net (fast)'])
+        self.NetAvg.setFont(self.medfont)
+        self.NetAvg.setToolTip('average 4 different fit networks or run 1 network to <i>increase</i> run speed')
+        self.l0.addWidget(self.NetAvg, b,1,1,1)
 
         b+=1
         # choose models
@@ -412,6 +421,7 @@ class MainW(QtGui.QMainWindow):
         label.setStyleSheet(label_style)
         label.setFont(self.medfont)
         label.setToolTip('there is a <em>cytoplasm</em> model and a <em>nuclei</em> model, choose what you want to segment')
+        self.ModelChoose.setToolTip('there is a <em>cytoplasm</em> model and a <em>nuclei</em> model, choose what you want to segment')
         self.l0.addWidget(label, b, 0,1,1)
 
         b+=1
@@ -429,8 +439,10 @@ class MainW(QtGui.QMainWindow):
             label.setFont(self.medfont)
             if i==0:
                 label.setToolTip('this is the channel in which the cytoplasm or nuclei exist that you want to segment')
+                self.ChannelChoose[i].setToolTip('this is the channel in which the cytoplasm or nuclei exist that you want to segment')
             else:
                 label.setToolTip('if <em>cytoplasm</em> model is chosen, and you also have a nuclear channel, then choose the nuclear channel for this option')
+                self.ChannelChoose[i].setToolTip('if <em>cytoplasm</em> model is chosen, and you also have a nuclear channel, then choose the nuclear channel for this option')
             self.l0.addWidget(label, b, 0,1,1)
             self.l0.addWidget(self.ChannelChoose[i], b, 1,1,1)
             b+=1
@@ -458,8 +470,8 @@ class MainW(QtGui.QMainWindow):
         # post-hoc paramater tuning
 
         b+=1
-        label = QtGui.QLabel('flow error threshold:')
-        label.setToolTip('threshold on flow error to accept for masks (set higher to get more cells)')
+        label = QtGui.QLabel('model match threshold:')
+        label.setToolTip('threshold on gradient match to accept a mask (set lower to get more cells)')
         label.setStyleSheet(label_style)
         label.setFont(self.medfont)
         self.l0.addWidget(label, b, 0,1,2)
@@ -470,7 +482,7 @@ class MainW(QtGui.QMainWindow):
         self.threshslider.setOrientation(QtCore.Qt.Horizontal)
         self.threshslider.setMinimum(1.0)
         self.threshslider.setMaximum(30.0)
-        self.threshslider.setValue(4)
+        self.threshslider.setValue(31 - 4)
         self.l0.addWidget(self.threshslider, b, 0,1,2)
         self.threshslider.valueChanged.connect(self.compute_cprob)
         self.threshslider.setStyleSheet(guiparts.horizontal_slider_style())
@@ -686,6 +698,9 @@ class MainW(QtGui.QMainWindow):
             if self.ncells> 0:
                 self.remove_cell(self.ncells)
 
+    def undo_remove_action(self):
+        self.undo_remove_cell()
+
     def get_files(self):
         images = []
         images.extend(glob.glob(os.path.dirname(self.filename) + '/*.png'))
@@ -772,6 +787,8 @@ class MainW(QtGui.QMainWindow):
         self.p0.addItem(self.layer)
         self.p0.addItem(self.scale)
 
+        guiparts.make_quadrants(self)
+
     def reset(self):
         # ---- start sets of points ---- #
         self.selected = 0
@@ -786,6 +803,7 @@ class MainW(QtGui.QMainWindow):
         self.stroke_appended = True
         self.ncells = 0
         self.zdraw = []
+        self.removed_cell = []
         self.cellcolors = [np.array([255,255,255])]
         # -- set menus to default -- #
         self.color = 0
@@ -870,7 +888,6 @@ class MainW(QtGui.QMainWindow):
     def remove_cell(self, idx):
         # remove from manual array
         self.selected = 0
-        self.ismanual = np.delete(self.ismanual, idx-1)
         for z in range(self.NZ):
             cp = self.cellpix[z]==idx
             op = self.outpix[z]==idx
@@ -883,6 +900,11 @@ class MainW(QtGui.QMainWindow):
             self.cellpix[z, self.cellpix[z]>idx] -= 1
             self.outpix[z, self.outpix[z]>idx] -= 1
         self.update_plot()
+        if self.NZ==1:
+            self.removed_cell = [self.ismanual[idx-1], self.cellcolors[idx], np.nonzero(cp), np.nonzero(op)]
+            self.redo.setEnabled(True)
+        # remove cell from lists
+        self.ismanual = np.delete(self.ismanual, idx-1)
         del self.cellcolors[idx]
         del self.zdraw[idx-1]
         self.ncells -= 1
@@ -892,6 +914,25 @@ class MainW(QtGui.QMainWindow):
         if self.NZ==1:
             io._save_sets(self)
 
+    def undo_remove_cell(self):
+        if len(self.removed_cell) > 0:
+            z = 0
+            ar, ac = self.removed_cell[2]
+            vr, vc = self.removed_cell[3]
+            color = self.removed_cell[1]
+            self.draw_mask(z, ar, ac, vr, vc, color)
+            self.toggle_mask_ops()
+            self.cellcolors.append(color)
+            self.ncells+=1
+            self.ismanual = np.append(self.ismanual, self.removed_cell[0])
+            self.zdraw.append([])
+            print('added back removed cell')
+            self.update_plot()
+            io._save_sets(self)
+            self.removed_cell = []
+            self.redo.setEnabled(False)
+
+
     def remove_stroke(self, delete_points=True):
         #self.current_stroke = get_unique_points(self.current_stroke)
         stroke = np.array(self.strokes[-1])
@@ -998,6 +1039,7 @@ class MainW(QtGui.QMainWindow):
                 color = self.colormap[col_rand,:3]
                 median = self.add_mask(points=self.current_point_set, color=color)
                 if median is not None:
+                    self.removed_cell = []
                     self.toggle_mask_ops()
                     self.cellcolors.append(color)
                     self.ncells+=1
@@ -1071,7 +1113,7 @@ class MainW(QtGui.QMainWindow):
                     mask[ar[ioverlap], ac[ioverlap]] = 0
                     ar, ac = ar[~ioverlap], ac[~ioverlap]
                 # compute outline of mask
-                outlines = plot.masks_to_outlines(mask)
+                outlines = utils.masks_to_outlines(mask)
                 vr, vc = np.nonzero(outlines)
                 vr, vc = vr+ymin, vc+xmin
                 ar, ac = ar+ymin, ac+xmin
@@ -1173,9 +1215,9 @@ class MainW(QtGui.QMainWindow):
         if self.cellprob != self.probslider.value():
             rerun = True
             self.cellprob = self.probslider.value()
-        if self.threshold != self.threshslider.value()/10.:
+        if self.threshold != (31 - self.threshslider.value())/10.:
             rerun = True
-            self.threshold = self.threshslider.value()/10.
+            self.threshold = (31 - self.threshslider.value())/10.
         if not rerun:
             return
         
@@ -1190,8 +1232,9 @@ class MainW(QtGui.QMainWindow):
         maski = dynamics.get_masks(self.flows[3].copy(), iscell=(self.flows[4][-1]>self.cellprob),
                                     flows=self.flows[4][:-1], threshold=thresh)
         if self.NZ==1:
-            maski = dynamics.fill_holes(maski)
-
+            maski = utils.fill_holes_and_remove_small_masks(maski)
+        maski = transforms.resize_image(maski, self.cellpix.shape[-2], self.cellpix.shape[-1],
+                                        interpolation=cv2.INTER_NEAREST)
         self.masksOn = True
         self.outlinesOn = True
         self.MCheckBox.setChecked(True)
@@ -1221,11 +1264,11 @@ class MainW(QtGui.QMainWindow):
             channels = self.get_channels()
             self.diameter = float(self.Diameter.text())
             try:
-                augment = (not self.FastMode.isChecked())
+                net_avg = 1 - self.NetAvg.currentIndex()
                 masks, flows, _, _ = self.model.eval(data, channels=channels,
-                                                diameter=self.diameter, invert=self.invert.isChecked(),
-                                                net_avg=augment, augment=augment,
-                                                do_3D=do_3D, progress=self.progress)
+                                                    diameter=self.diameter, invert=self.invert.isChecked(),
+                                                    net_avg=net_avg, augment=False,
+                                                    do_3D=do_3D, progress=self.progress)
             except Exception as e:
                 print('NET ERROR: %s'%e)
                 self.progress.setValue(0)
@@ -1238,13 +1281,18 @@ class MainW(QtGui.QMainWindow):
             #    flows = flows[0]
             if not do_3D:
                 masks = masks[np.newaxis,...]
-            self.flows[0] = flows[0]
+            self.flows[0] = transforms.resize_image(flows[0].copy(), masks.shape[-2], masks.shape[-1],
+                                                    interpolation=cv2.INTER_NEAREST)
             self.flows[1] = (np.clip(utils.normalize99(flows[2]),0,1) * 255).astype(np.uint8)
+            self.flows[1] = transforms.resize_image(self.flows[1], masks.shape[-2], masks.shape[-1],
+                                                    interpolation=cv2.INTER_NEAREST)
             if not do_3D:
-                self.flows[2] = np.zeros(flows[1][0].shape, dtype=np.uint8)
+                self.flows[2] = np.zeros(masks.shape[1:], dtype=np.uint8)
                 self.flows = [self.flows[n][np.newaxis,...] for n in range(len(self.flows))]
             else:
                 self.flows[2] = (flows[1][0]/10 * 127 + 127).astype(np.uint8)
+                self.flows[2] = transforms.resize_image(self.flows[2], masks.shape[-2], masks.shape[-1],
+                                                        interpolation=cv2.INTER_NEAREST)
             if len(flows)>2:
                 self.flows.append(flows[3])
                 self.flows.append(np.concatenate((flows[1], flows[2][np.newaxis,...]), axis=0))
