@@ -1,6 +1,5 @@
 from PyQt5 import QtGui, QtCore, Qt, QtWidgets
-
-from . import io
+from . import io, utils
 
 def mainmenu(parent):
     main_menu = parent.menuBar()
@@ -59,6 +58,12 @@ def editmenu(parent):
     parent.undo.setEnabled(False)
     edit_menu.addAction(parent.undo)
 
+    parent.redo = QtGui.QAction('Undo remove mask', parent)
+    parent.redo.setShortcut("Ctrl+Y")
+    parent.redo.triggered.connect(parent.undo_remove_action)
+    parent.redo.setEnabled(False)
+    edit_menu.addAction(parent.redo)
+
     parent.ClearButton = QtGui.QAction('Clear all masks', parent)
     parent.ClearButton.setShortcut("Ctrl+0")
     parent.ClearButton.triggered.connect(parent.clear_all)
@@ -74,7 +79,17 @@ def editmenu(parent):
 def helpmenu(parent):
     main_menu = parent.menuBar()
     help_menu = main_menu.addMenu("&Help")
+    
+    checkMKL = QtGui.QAction("Check CPU mxnet-mkl -- see terminal", parent)
+    checkMKL.triggered.connect(utils.check_mkl)
+    help_menu.addAction(checkMKL)
+
     openHelp = QtGui.QAction("&Help window", parent)
     openHelp.setShortcut("Ctrl+H")
     openHelp.triggered.connect(parent.help_window)
     help_menu.addAction(openHelp)
+
+    openGUI = QtGui.QAction("&GUI layout", parent)
+    openGUI.setShortcut("Ctrl+G")
+    openGUI.triggered.connect(parent.gui_window)
+    help_menu.addAction(openGUI)
