@@ -2,6 +2,125 @@ import numpy as np
 from . import utils, dynamics
 from numba import jit
 from scipy.optimize import linear_sum_assignment
+from scipy.ndimage import convolve
+
+
+def mask_ious(masks_true, masks_pred):
+    """ return best-matched masks """
+    iou = _intersection_over_union(masks_true, masks_pred)[1:,1:]
+    n_min = min(iou.shape[0], iou.shape[1])
+    costs = -(iou >= 0.5).astype(float) - iou / (2*n_min)
+    true_ind, pred_ind = linear_sum_assignment(costs)
+    iout = np.zeros(masks_true.max())
+    iout[true_ind] = iou[true_ind,pred_ind]
+    preds = np.zeros(masks_true.max(), 'int')
+    preds[true_ind] = pred_ind+1
+    return iout, preds
+
+def boundary_scores(masks_true, masks_pred, scales):
+    """ boundary precision / recall / Fscore """
+    diams = [utils.diameters(lbl)[0] for lbl in masks_true]
+    precision = np.zeros((len(scales), len(masks_true)))
+    recall = np.zeros((len(scales), len(masks_true)))
+    fscore = np.zeros((len(scales), len(masks_true)))
+    for j, scale in enumerate(scales):
+        for n in range(len(masks_true)):
+            diam = max(1, scale * diams[n])
+            rs, ys, xs = utils.circleMask([int(np.ceil(diam)), int(np.ceil(diam))])
+            filt = (rs <= diam).astype(np.float32)
+            otrue = utils.masks_to_outlines(masks_true[n])
+            otrue = convolve(otrue, filt)
+            opred = utils.masks_to_outlines(masks_pred[n])
+            opred = convolve(opred, filt)
+            tp = np.logical_and(otrue==1, opred==1).sum()
+            fp = np.logical_and(otrue==0, opred==1).sum()
+            fn = np.logical_and(otrue==1, opred==0).sum()
+            precision[j,n] = tp / (tp + fp)
+            recall[j,n] = tp / (tp + fn)
+        fscore[j] = 2 * precision[j] * recall[j] / (precision[j] + recall[j])
+    return precision, recall, fscore
+
+
+def aggregated_jaccard_index(masks_true, masks_pred):
+    """ AJI = intersection of all matched masks / union of all masks 
+    
+    Parameters
+    ------------
+    
+    masks_true: list of ND-arrays (int) or ND-array (int) 
+        where 0=NO masks; 1,2... are mask labels
+    masks_pred: list of ND-arrays (int) or ND-array (int) 
+        ND-array (int) where 0=NO masks; 1,2... are mask labels
+
+    Returns
+    ------------
+
+    aji : aggregated jaccard index for each set of masks
+
+    """
+
+    aji = np.zeros(len(masks_true))
+    for n in range(len(masks_true)):
+        iout, preds = mask_ious(masks_true[n], masks_pred[n])
+        inds = np.arange(0, masks_true[n].max(), 1, int)
+        overlap = _label_overlap(masks_true[n], masks_pred[n])
+        union = np.logical_or(masks_true[n]>0, masks_pred[n]>0).sum()
+        overlap = overlap[inds[preds>0]+1, preds[preds>0].astype(int)]
+        aji[n] = overlap.sum() / union
+    return aji 
+
+
+def average_precision(masks_true, masks_pred, threshold=[0.5, 0.75, 0.9]):
+    """ average precision estimation: AP = TP / (TP + FP + FN)
+
+    This function is based heavily on the *fast* stardist matching functions
+    (https://github.com/mpicbg-csbd/stardist/blob/master/stardist/matching.py)
+
+    Parameters
+    ------------
+    
+    masks_true: list of ND-arrays (int) or ND-array (int) 
+        where 0=NO masks; 1,2... are mask labels
+    masks_pred: list of ND-arrays (int) or ND-array (int) 
+        ND-array (int) where 0=NO masks; 1,2... are mask labels
+
+    Returns
+    ------------
+
+    ap: array [len(masks_true) x len(threshold)]
+        average precision at thresholds
+    tp: array [len(masks_true) x len(threshold)]
+        number of true positives at thresholds
+    fp: array [len(masks_true) x len(threshold)]
+        number of false positives at thresholds
+    fn: array [len(masks_true) x len(threshold)]
+        number of false negatives at thresholds
+
+    """
+    not_list = False
+    if not isinstance(masks_true, list):
+        masks_true = [masks_true]
+        masks_pred = [masks_pred]
+        not_list = True
+    if not isinstance(threshold, list) and not isinstance(threshold, np.ndarray):
+        threshold = [threshold]
+    ap  = np.zeros((len(masks_true), len(threshold)), np.float32)
+    tp  = np.zeros((len(masks_true), len(threshold)), np.float32)
+    fp  = np.zeros((len(masks_true), len(threshold)), np.float32)
+    fn  = np.zeros((len(masks_true), len(threshold)), np.float32)
+    n_true = np.array(list(map(np.max, masks_true)))
+    n_pred = np.array(list(map(np.max, masks_pred)))
+    for n in range(len(masks_true)):
+        #_,mt = np.reshape(np.unique(masks_true[n], return_index=True), masks_pred[n].shape)
+        iou = _intersection_over_union(masks_true[n], masks_pred[n])[1:, 1:]
+        for k,th in enumerate(threshold):
+            tp[n,k] = _true_positive(iou, th)
+        fp[n] = n_pred[n] - tp[n]
+        fn[n] = n_true[n] - tp[n]
+        ap[n] = tp[n] / (tp[n] + fp[n] + fn[n])
+    if not_list:
+        ap, tp, fp, fn = ap[0], tp[0], fp[0], fn[0]
+    return ap, tp, fp, fn
 
 @jit(nopython=True)
 def _label_overlap(x, y):
@@ -51,6 +170,7 @@ def _intersection_over_union(masks_true, masks_pred):
     n_pixels_pred = np.sum(overlap, axis=0, keepdims=True)
     n_pixels_true = np.sum(overlap, axis=1, keepdims=True)
     iou = overlap / (n_pixels_pred + n_pixels_true - overlap)
+    iou[np.isnan(iou)] = 0.0
     return iou
 
 def _true_positive(iou, th):
@@ -78,57 +198,6 @@ def _true_positive(iou, th):
     tp = match_ok.sum()
     return tp
 
-def average_precision(masks_true, masks_pred, threshold=[0.5, 0.75, 0.9]):
-    """ average precision estimation: AP = TP / (TP + FP + FN)
-
-    This function is based heavily on the *fast* stardist matching functions
-    (https://github.com/mpicbg-csbd/stardist/blob/master/stardist/matching.py)
-
-    Parameters
-    ------------
-    
-    masks_true: list of ND-arrays (int) or ND-array (int) 
-        where 0=NO masks; 1,2... are mask labels
-    masks_pred: list of ND-arrays (int) or ND-array (int) 
-        ND-array (int) where 0=NO masks; 1,2... are mask labels
-
-    Returns
-    ------------
-
-    ap: array [len(masks_true) x len(threshold)]
-        average precision at thresholds
-    tp: array [len(masks_true) x len(threshold)]
-        number of true positives at thresholds
-    fp: array [len(masks_true) x len(threshold)]
-        number of false positives at thresholds
-    fn: array [len(masks_true) x len(threshold)]
-        number of false negatives at thresholds
-
-    """
-    not_list = False
-    if not isinstance(masks_true, list):
-        masks_true = [masks_true]
-        masks_pred = [masks_pred]
-        not_list = True
-    if not isinstance(threshold, list) and not isinstance(threshold, np.ndarray):
-        threshold = [threshold]
-    ap  = np.zeros((len(masks_true), len(threshold)), np.float32)
-    tp  = np.zeros((len(masks_true), len(threshold)), np.float32)
-    fp  = np.zeros((len(masks_true), len(threshold)), np.float32)
-    fn  = np.zeros((len(masks_true), len(threshold)), np.float32)
-    n_true = np.array(list(map(np.max, masks_true)))
-    n_pred = np.array(list(map(np.max, masks_pred)))
-    for n in range(len(masks_true)):
-        iou = _intersection_over_union(masks_true[n], masks_pred[n])[1:, 1:]
-        for k,th in enumerate(threshold):
-            tp[n,k] = _true_positive(iou, th)
-        fp[n] = n_pred[n] - tp[n]
-        fn[n] = n_true[n] - tp[n]
-        ap[n] = tp[n] / (tp[n] + fp[n] + fn[n])
-    if not_list:
-        ap, tp, fp, fn = ap[0], tp[0], fp[0], fn[0]
-    return ap, tp, fp, fn
-
 def flow_error(maski, dP_net):
     """ error in flows from predicted masks vs flows predicted by network run on image
 
