@@ -57,13 +57,30 @@ class resdown(nn.HybridBlock):
         x = x + self.conv[3](self.conv[2](x))
         return x
 
+class convdown(nn.HybridBlock):
+    def __init__(self, nconv, **kwargs):
+        super(convdown, self).__init__(**kwargs)
+        with self.name_scope():
+            self.conv = nn.HybridSequential()
+            for t in range(2):
+                self.conv.add(batchconv(nconv, 3))
+
+    def hybrid_forward(self, F, x):
+        x = self.conv[0](x)
+        x = self.conv[1](x)
+        return x
+
+
 class downsample(nn.HybridBlock):
-    def __init__(self, nbase, **kwargs):
+    def __init__(self, nbase, residual_on=True, **kwargs):
         super(downsample, self).__init__(**kwargs)
         with self.name_scope():
             self.down = nn.HybridSequential()
             for n in range(len(nbase)):
-                self.down.add(resdown(nbase[n]))
+                if residual_on:
+                    self.down.add(resdown(nbase[n]))
+                else:
+                    self.down.add(convdown(nbase[n]))
 
     def hybrid_forward(self, F, x):
         xd = []
@@ -76,41 +93,67 @@ class downsample(nn.HybridBlock):
         return xd
 
 class batchconvstyle(nn.HybridBlock):
-    def __init__(self, nconv, **kwargs):
+    def __init__(self, nconv, concatenation=False, **kwargs):
         super(batchconvstyle, self).__init__(**kwargs)
         with self.name_scope():
             self.conv = batchconv(nconv, 3)
-            self.full = nn.Dense(nconv)
+            if concatenation:
+                self.full = nn.Dense(nconv*2)
+            else:
+                self.full = nn.Dense(nconv)
+            self.concatenation = concatenation
 
-    def hybrid_forward(self, F, style, x):
+    def hybrid_forward(self, F, style, x, y=None):
+        if y is not None:
+            if self.concatenation:
+                x = F.concat(y, x, dim=1)
+            else:
+                x = x + y
         feat = self.full(style)
         y = F.broadcast_add(x, feat.expand_dims(-1).expand_dims(-1))
         y = self.conv(y)
         return y
 
+class convup(nn.HybridBlock):
+    def __init__(self, nconv, concatenation=False, **kwargs):
+        super(convup, self).__init__(**kwargs)
+        with self.name_scope():
+            self.conv = nn.HybridSequential()
+            self.conv.add(batchconv(nconv, 3))
+            self.conv.add(batchconvstyle(nconv, concatenation))
+            
+    def hybrid_forward(self, F, x, y, style):
+        x = self.conv[0](x)
+        x = self.conv[1](style, x, y)
+        return x
+
+
 class resup(nn.HybridBlock):
-    def __init__(self, nconv, **kwargs):
+    def __init__(self, nconv, concatenation=False, **kwargs):
         super(resup, self).__init__(**kwargs)
         with self.name_scope():
             self.conv = nn.HybridSequential()
             self.conv.add(batchconv(nconv,3))
-            for n in range(3):
-                self.conv.add(batchconvstyle(nconv))
+            self.conv.add(batchconvstyle(nconv, concatenation))
+            self.conv.add(batchconvstyle(nconv))
+            self.conv.add(batchconvstyle(nconv))
             self.proj  = batchconv0(nconv, 1)
-            #self.proj2  = batchconv0(nconv, 1)
 
     def hybrid_forward(self, F, x, y, style):
-        x = self.proj(x) + self.conv[1](style, self.conv[0](x) + y)
+        x = self.proj(x) + self.conv[1](style, self.conv[0](x), y)
         x = x + self.conv[3](style, self.conv[2](style, x))
         return x
 
 class upsample(nn.HybridBlock):
-    def __init__(self, nbase, **kwargs):
+    def __init__(self, nbase, residual_on=True, concatenation=False, **kwargs):
         super(upsample, self).__init__(**kwargs)
         with self.name_scope():
             self.up = nn.HybridSequential()
             for n in range(len(nbase)):
-                self.up.add(resup(nbase[n]))
+                if residual_on:
+                    self.up.add(resup(nbase[n], concatenation=concatenation))
+                else:
+                    self.up.add(convup(nbase[n], concatenation=concatenation))
 
     def hybrid_forward(self, F, style, xd):
         x= self.up[-1](xd[-1], xd[-1], style)
@@ -134,20 +177,24 @@ class make_style(nn.HybridBlock):
         return style
 
 class CPnet(gluon.HybridBlock):
-    def __init__(self, nbase, nout, **kwargs):
+    def __init__(self, nbase, nout, residual_on=True, style_on=True, concatenation=False, **kwargs):
         super(CPnet, self).__init__(**kwargs)
         with self.name_scope():
-            #self.conv1 = convbatchrelu(16, 3)
-            self.downsample = downsample(nbase)
-            self.upsample = upsample(nbase)
+            self.nbase = nbase
+            self.downsample = downsample(nbase, residual_on=residual_on)
+            self.upsample = upsample(nbase, residual_on=residual_on, concatenation=concatenation)
             self.output = batchconv(nout, 1)
             self.make_style = make_style()
+            self.style_on = style_on
 
     def hybrid_forward(self, F, data):
         #data     = self.conv1(data)
         T0    = self.downsample(data)
         style = self.make_style(T0[-1])
+        style0 = style
+        if not self.style_on:
+            style = style * 0 
         T0    = self.upsample(style, T0)
         T0    = self.output(T0)
 
-        return T0, style
+        return T0, style0
